/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Application
    costFunction

Description
    compute the Lighthill stress tensor

\*---------------------------------------------------------------------------*/

#include "fvCFD.H"
#include "cellSet.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{
    #include "setRootCase.H"

    #include "createTime.H"
    #include "createMesh.H"
    #include "createFields.H"

    // * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

    Info<< "Compute the tensor: velocity gradient grad(U)!\n" << endl;
    volTensorField gradU(fvc::grad(U));
    Info<< "Compute the vector: divergence ofvelocity gradient div(grad(U))!\n" << endl;
    volVectorField divGradU(fvc::div(gradU));

    Info<< "Compute the scalar: div(grad(U)) & U!\n" << endl;
    /*volScalarField Tij(
	IOobject
	(
		"Tij",
		runTime.timeName(),
		mesh,
		IOobject::NO_READ,
		IOobject::AUTO_WRITE
	),
	2*(divGradU & U) + 
	sqr(gradU.component(tensor::XX)) + sqr(gradU.component(tensor::YY)) + sqr(gradU.component(tensor::ZZ)) + 
	2*(gradU.component(tensor::YX)*gradU.component(tensor::XY) + gradU.component(tensor::ZY)*gradU.component(tensor::YZ) + gradU.component(tensor::XZ)*gradU.component(tensor::ZX))

	//d2(u_i*u_j)/dx_i/dx_j = 2*div(grad(U))*U+du_i/dx_j*du_j/dx_i
    );*/
    volScalarField Tij(
	IOobject
	(
		"Tij",
		runTime.timeName(),
		mesh,
		IOobject::NO_READ,
		IOobject::AUTO_WRITE
	),
	(mesh.C() & mesh.C())*0.0
    );
/* //this is done now with sets
    scalar xmin = -0.015;
    scalar xmax = 0.035;
    scalar ymin = -0.015;
    scalar ymax = 0.035;
    scalar zmin = -0.0;
    scalar zmax = 0.02;
    
    forAll(Tij,cellI)
    {
	vector centroid = Tij.mesh().C()[cellI];
	if(centroid[0] >= xmin &&
		centroid[0] <= xmax &&
		centroid[1] >= ymin &&
		centroid[1] <= ymax &&
		centroid[2] >= zmin &&
		centroid[2] <= zmax)
		
		Tij[cellI] = 0.0;
		2*(divGradU[cellI] & U[cellI]) + 
		sqr(gradU[cellI].component(tensor::XX)) + sqr(gradU[cellI].component(tensor::YY)) + sqr(gradU[cellI].component(tensor::ZZ)) + 
		2*(gradU[cellI].component(tensor::YX)*gradU[cellI].component(tensor::XY) + gradU[cellI].component(tensor::ZY)*gradU[cellI].component(tensor::YZ) + 
		gradU[cellI].component(tensor::XZ)*gradU[cellI].component(tensor::ZX));
    }
*/
/*    word patchName = "outlet";
    label patchID = mesh.boundaryMesh().findPatchID(patchName);
    error err("Error!\n");
    if(-1 == patchID)
	err.exit();
    const fvPatch& cPatch = mesh.boundary()[patchID];
    const labelUList& cellsPatch = cPatch.faceCells();
    forAll(cellsPatch,cellI)
	Tij[cellsPatch[cellI]] = 1.0;

    patchName = "innerBox";
    patchID = mesh.boundaryMesh().findPatchID(patchName);
    if(-1 == patchID)
	err.exit();
    const fvPatch& cPatch1 = mesh.boundary()[patchID];
    const labelUList& cellsPatch1 = cPatch1.faceCells();
    forAll(cellsPatch1,cellI)
	Tij[cellsPatch1[cellI]] = 1.0;
*/
    scalar volume = 0.0;
    cellSet cSet(mesh,"box");
    const labelList& cells = cSet.toc(); 
    forAll(cells,cellI)
    {
	Tij[cells[cellI]] =
	2*(divGradU[cells[cellI]] & U[cells[cellI]]) + 
	sqr(gradU[cells[cellI]].component(tensor::XX)) + sqr(gradU[cells[cellI]].component(tensor::YY)) + sqr(gradU[cells[cellI]].component(tensor::ZZ)) + 
	2*(gradU[cells[cellI]].component(tensor::YX)*gradU[cells[cellI]].component(tensor::XY) + gradU[cells[cellI]].component(tensor::ZY)*gradU[cells[cellI]].component(tensor::YZ) + 
	gradU[cells[cellI]].component(tensor::XZ)*gradU[cells[cellI]].component(tensor::ZX));
	volume += mesh.V()[cells[cellI]];
    }

    Tij.write();

    Info<< "Number of cells in box: " << cSet.size() << endl;

    Info<< "Integrate Tij in the whole volume!\n" << endl;
    scalar intTij = gSum(Tij.internalField() * mesh.V());
	reduce(volume,sumOp<scalar>());
	
    Info<< "The cost function has the value of: " << intTij/(volume+SMALL) << " [1/s2]\nDone!\n" << endl;

    return 0;
}


// ************************************************************************* //
