/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011-2013 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Application
    moveMesh

Description
    Solver for moving meshes.

\*---------------------------------------------------------------------------*/


#include "argList.H"
//#include "fvIOoptionList.H"
#include "simpleControl.H"
#include "Time.H"
#include "dynamicFvMesh.H"

#include "motionSolver.H"
#include "velocityMotionSolver.H"
#include "primitivePatchInterpolation.H"

#include "primitiveMesh.H"
#include "primitivePatch.H"

#include "pointMesh.H" //Perhaps not needed..?
#include "pointFields.H" //Perhaps not needed..?
#include "pointPatchField.H"
#include "volPointInterpolation.H"
#include "fixedValuePointPatchField.H"


#include "fvcSmooth.H"

#include "bound.H"


using namespace Foam;

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{

#   include "setRootCase.H"
#   include "createTime.H"
#   include "createMesh.H"
#   include "createFields.H"

    autoPtr<motionSolver> motionPtr = motionSolver::New(mesh);
    	
	word patchName = "innerBox";
	label patchID = mesh.boundaryMesh().findPatchID(patchName);

	//Info << "PatchName = " << patchName << " has ID = " << patchID << endl;
	
	pointVectorField& PointDisplacement = const_cast<pointVectorField&>
	(
		mesh.objectRegistry::lookupObject<pointVectorField>
		(
			"pointDisplacement"
		)
	);
	
	//Get the vector field of the patch
	vectorField &pDisp=refCast<vectorField>(PointDisplacement.boundaryField()[patchID]);

	//Find the relevant size of the vector and declare a vectorField.
	int Psize= pDisp.size();
	vectorField dispVals(Psize);
	
	// ########################################################################## //

	//- set-up interpolator
	primitivePatchInterpolation patchInterpolator( mesh.boundaryMesh()[patchID] );

	// displacement based on the external calculation (must provide field data in 0/)
	scalarField sensitivityPatch = sensitivity.boundaryField()[patchID];

	//- perform interpolation
	scalarField faceValues = patchInterpolator.faceToPointInterpolate(sensitivityPatch);
	
	vectorField &PointPointer = refCast<vectorField>(PointDisplacement.boundaryField()[patchID]);
	vectorField PointNormalVector = mesh.boundaryMesh()[patchID].pointNormals();

	// loop over points to move the nodes
	forAll(dispVals, index)
	{
		dispVals[index].x() = PointPointer[index].x() - alpha.value() * faceValues[index] * PointNormalVector[index].x();
		dispVals[index].y() = PointPointer[index].y() - alpha.value() * faceValues[index] * PointNormalVector[index].y();
		dispVals[index].z() = PointPointer[index].z() - alpha.value() * faceValues[index] * PointNormalVector[index].z();
	}

	// ########################################################################## //

	//Once the values have been assigned to dispVals, assign them to cellDisplacement boundaryField
	PointDisplacement.boundaryField()[patchID] == dispVals;
	
	
    while (runTime.loop())
    {
        Info<< "Time = " << runTime.timeName() << endl;
        
        mesh.movePoints(motionPtr->newPoints());

        runTime.write();

        Info<< "ExecutionTime = " << runTime.elapsedCpuTime() << " s"
            << "  ClockTime = " << runTime.elapsedClockTime() << " s"
            << nl << endl;
    }

    Info<< "End\n" << endl;

    return 0;
}


// ************************************************************************* //
